"""Handles drawing to the two windows (thread window and empire window)"""

import pygame
import data
import random
import math
import characteristics
import Numeric as N

map = None #Map window (on top)
threads = None #Threads window (on bottom)
OCEAN_COLOR = (181, 208, 208)
SEALANE_COLOR = (185, 209, 209)
ARCTIC_CIRCLE = 75 #y-coordinate above which new empires cannot spawn (not actually geographic arctic circle)
NORTHERN_LINE = 100 #y-coordinate at which growth slows down due to cold
SOUTHERN_LINE = 310 #y-coordinate below which new empires cannot spawn (corresponds to highest point of Antarctica)

ending = None #Will be one of the following once game ends
NUKE_VICTORY = 1
CONQUEST_VICTORY = 2
DEFEAT = 3

TECH_LINE = ((400, 'cavalry'), (400, 'stirrups'), (300, 'composite bow'), (600, 'steel'), (700, 'crossbow'), (300, 'catapult'), (600, 'gunpowder'), (200, 'cannon forging'), (400, 'mass conscription'), (300, 'replaceable parts'), (200, 'assembly line'), (200, 'motorization'), (200, 'flight'), (200, 'composites'), (200, 'nuclear weaponry'), (100, 'NUCLEAR ANNIHILATION')) #Each tech increases soldierstr by 0.05. The format is (cost, tech name)

def init(screen, mapimg):
	dim = screen.get_rect()
	w, h = dim.size
	maph = mapimg.get_rect().h
	#Ready the windows
	global map, threads
	map = Map(w, maph, mapimg, screen)
	threads = Threads(w, h-maph, maph, screen)
	return map, threads

class Map(pygame.sprite.Group):
	
	def __init__(self, w, h, mapimg, screen):
		pygame.sprite.Group.__init__(self)
		self.map = mapimg
		self.overlay = self.map.copy()
		self.nationcolors = dict() #key = color, value = Country object
		self.smallfont = pygame.font.Font(None, 14)
		self.newsfont = pygame.font.Font(None, 15)
		self.news = [] #Latest in international developments, printed largely on the world (list of surfaces)
		#Define where to draw this window
		self.dest = screen.subsurface(self.map.get_rect())
		#Draw background
		self.dest.blit(self.map, (0, 0))
		pygame.display.flip()

	def populate(self):
		#Define countries
		self.add(Country((270, 141), pygame.Color('orange')[:3], True)) #Player country
		self.add(Country((280, 141), pygame.Color('purple')[:3]))
		self.add(Country((404, 156), pygame.Color('pink')[:3]))
		self.add(Country((279, 156), pygame.Color('yellow')[:3]))
		self.add(Country((117, 186), pygame.Color('green')[:3]))
		self.add(Country((131, 220), pygame.Color('brown')[:3]))
		self.add(Country((345, 173), pygame.Color('blue')[:3]))

	def draw(self):
		self.dest.blit(self.overlay, (0, 0))
		#pygame.sprite.Group.draw(self, self.dest)
		#Flag coordinates and dimensions
		x = 1
		y = 1
		w = 16 #Golden ratio approximation
		h = 10
		countries = self.sprites()
		countries.sort(lambda x, y: -x.player)
		#Draw stats
		for country in countries:
			textx = x + w + 5 #Five pixels to the right of flag
			#Draw flag:
			pygame.draw.rect(self.dest, country.color, (x, y, w, h))
			chars = country.characteristics.items()
			#Sort characteristics by value
			chars.sort(lambda x, y: (y[1] - x[1], x[1] - y[1])[x[1] < 0 and y[1] < 0])
			if country.player:#Display all values
				for char, val in chars:
					textstr = char+': '+str(val)+'%'
					text = self.smallfont.render(textstr, 1, (0, 0, 0))
					self.dest.blit(text, (textx, y))
					y += text.get_rect().h #Move y down by height of text
				#Show research
				textstr = 'Research: '+str(int(country.tech/TECH_LINE[country.techi][0]*100))+'%'
				text = self.smallfont.render(textstr, 1, (0, 0, 0))
				self.dest.blit(text, (textx, y))
				y += text.get_rect().h #Move y down by height of text
				country.characteristics = threads.characteristics #Update characteristics (for next draw, which doesn't affect gameplay since the draws are so fast)
			elif len(chars) >= 2: #Display most significant values
				#Display highest value
				char, val = chars[0]
				textstr = char+': '+str(val)+'%'
				text = self.smallfont.render(textstr, 1, (0, 0, 0))
				self.dest.blit(text, (textx, y))
				y += text.get_rect().h #Move y down by height of text
			#Draw area count, soldier strength
			text = self.smallfont.render('Tech level: '+str(country.soldierstr), 1, (0, 0, 0))
			self.dest.blit(text, (textx, y))

			y += h + 5 #Five pixel spacing between flags
		#Draw news
		x = 165 #Starting point for news
		y = 1
		for news in self.news[-5:]: #Draw last few news items
			self.dest.blit(news, (x, y))
			y += news.get_rect().h

	def tick(self):
		imga = pygame.surfarray.pixels3d(self.overlay)
		#Spawn new countries
		if random.randint(0, 100) <= 3: #Chances that a new empire will pop up
			x = random.randint(0, self.map.get_rect().w-1) #-1 for a zero-based index
			y = random.randint(ARCTIC_CIRCLE, SOUTHERN_LINE)
			color = imga[x, y]
			color = (int(color[0]), int(color[1]), int(color[2])) #deep copy of color
			if color != OCEAN_COLOR: #If it spawns in water, too bad. Ignore it
				success = True #True if empire is founded
				rebellion = False
				if color in self.nationcolors: #Spawns in nation. Make an administration check
					rebellion = True
					thresh = 50 #50% chance of success
					if 'administrative' in self.nationcolors[color].characteristics:
						thresh = max(0, thresh - self.nationcolors[color].characteristics['administrative']/2) #100% administrative rules out rebellion
					success = random.randint(0, 100) < thresh
				if success:
					#Pick color:
					badcolor = True #Initially true (for the do-while loop below), this will be false
					#or true depending on how close the color is to white, OCEAN_COLOR or a nation's color
					while badcolor:
						color = pygame.Color(random.choice(pygame.color.THECOLORS.keys()))
						r, g, b, a = color
						badcolor = math.sqrt(math.pow(r-OCEAN_COLOR[0], 2)+math.pow(g-OCEAN_COLOR[1], 2)+math.pow(b-OCEAN_COLOR[2], 2)) <= 50 or math.sqrt(math.pow(r - 255, 2)+math.pow(g - 255, 2)+math.pow(b - 255, 2)) <= 50
						for ncolor in self.nationcolors.keys():
							nr, ng, nb = ncolor
							badcolor = badcolor or math.sqrt(math.pow(r - nr, 2)+math.pow(g - ng, 2)+math.pow(b - nb, 2)) <= 50
					country = Country((x, y), color)
					if rebellion: #Give them a bit more guns
						if 'tactician' in country.characteristics:
							country.characteristics['tactician'] += 100
							country.characteristics['genius'] -= 100
					if rebellion:
						self.news.append(self.newsfont.render('A rebellion is taking place!', 1, color))
						color = imga[x, y]
						color = (int(color[0]), int(color[1]), int(color[2])) #deep copy of color
						country.techi = self.nationcolors[color].techi
					else:
						self.news.append(self.newsfont.render('A new empire arises!', 1, color))
						min_techi = 99
						for techstate in self: #Keep country up to date on technology
							min_techi = min(min_techi, techstate.techi)
						country.techi = min_techi
					self.add(country)
		#Let them spread
		for country in self:
			country.tick(imga)

		#Check for victory
		if len(self) == 1: #One player left
			if ending != DEFEAT and ending != NUKE_VICTORY: #If player hasn't been defeated or achieved alternate means of victory, they must have won
				ending = CONQUEST_VICTORY
		#Kill off countries
		for country in self:
			if len(country.border) == 0:
				self.news.append(self.newsfont.render('Defeat!', 1, country.color))
				if country.player:
					global ending
					ending = DEFEAT
				country.kill()

class Country(pygame.sprite.Sprite):

	def __init__(self, origin, color, player=False):
		"""player = whether this country is player-controlled"""
		pygame.sprite.Sprite.__init__(self)
		self.player = player
		if player:
			self.characteristics = threads.characteristics
			threads.addThread()
		else:
			self.characteristics = characteristics.createChar()
			for i in range(4): #Every country gets five random leaders (4 leaders here plus one above)
				self.characteristics = characteristics.addChars(self.characteristics, characteristics.createChar())
		self.border = set([origin]) #Actual border is not displayed, but areas inside border is
		self.image = map.overlay
		self.rect = self.image.get_rect()
		self.color = color[:3]
		self.borderColor = pygame.color.subtract(color, 30)[:3]
		map.nationcolors[self.color] = self
		map.nationcolors[self.borderColor] = self
		self.image.set_at(origin, self.borderColor)
		self.area = set()

		self.techi = 0 #Index along tech line (represents tech that is currently researching toward)
		self.tech = 0 #Amount of tech research done so far
		self.soldierstr = 1 #Soldier strength, base of 1. Used with tactician in combat
		self.starving = False #True when combat leaves logistics at 0 or below
		self.starvingnews = False #True when the player is informed about starving

	def tick(self, imga):
		memo = set() #Memoizing used pixels

		self_color = self.color
		self_borderColor = self.borderColor
		border_remove = self.border.discard
		border_append = self.border.add
		border = self.border
		memo_add = memo.add
		adj_range = (-1, 0, 1)
		maxx, maxy = self.image.get_rect().size
		maxx -= 1 #-1 to bring them to level with zero-based indices
		maxy -= 1
		nationcolors = map.nationcolors
		area = self.area
		area_add = self.area.add
		area_remove = self.area.discard
		self_chars = self.characteristics

		randints = [] #Set random integer list per tick
		for i in range(30):
			randints.append(random.randint(0, 100))
		randintc = 0 #counter for selecting random integer from list

		#Update tech:
		if 'genius' in self.characteristics and self.characteristics['genius'] > 0:
			self.tech += self.characteristics['genius']/10.0
			techreq, name = TECH_LINE[self.techi]
			if self.tech >= techreq:
				self.tech = self.tech - techreq #Overflow of tech
				map.news.append(map.newsfont.render('Tech discovery: '+name, 1, self.color))
				self.soldierstr += .05
				self.techi += 1
				if self.techi == len(TECH_LINE):
					global ending
					ending = NUKE_VICTORY
					self.techi -= 1 #TODO: Get rid of this after win condition is implemented
		#Make soldierstr a local variable after updating it
		soldierstr = self.soldierstr
		if self.starving:
			soldierstr *= 0.8
	
		#Consider chance:
		cons_crit = 50 #Base value, 50% chance of expansion
		if 'administrative' in self.characteristics:
			cons_crit = max(0, cons_crit + (self.characteristics['administrative']/8.0))

		#Logistics:
		logistics = len(border)/2 #Half of border may expand
		if 'macroeconomic' in self.characteristics:
			logistics = max(8, len(border)/10, logistics + logistics*(self.characteristics['macroeconomic']/100.0)) #At least one square will be expanded on (minimum of 8 squares for full one-pixel expansion)

		#Win battle chance:
		tact_crit = 0 #Base value, % chance of win
		if 'tactician' in self.characteristics:
			tact_crit = max(0, tact_crit + (self.characteristics['tactician']/2.0))

		#Do a clean sweep every five ticks:
		clean = False
		if threads.year%10 == 0:
			clean = True

		for pixel in tuple(border):
			x, y = pixel
			if imga[x, y] != self_color and imga[x, y] != self_borderColor: #Oopsies! Error correction
				#imga[x, y] = self_color
				border_remove(pixel)
				area_remove(pixel)
				color = imga[x, y]
				color = (int(color[0]), int(color[1]), int(color[2])) #deep copy of color
				if color in nationcolors:
					nationcolors[color].area.add(pixel)
				continue

			#Consider expansion (Don't skip even if you can't expand, because you still need to removed surrounded pixels)
			randintc += 1
			expansion = randints[randintc%30] <= cons_crit
			#Consider logistics
			expansion = expansion and logistics > 0
			#Expand if battle is won (sample random square to see if it's an unfriendly square)
			randintc += 1
			randx = (0, 1)[randints[randintc%30] < 50]
			randintc += 1
			randy = (0, 1)[randints[randintc%30] < 50]
			if x+randx > maxx:
				sample = imga[x+randx-maxx, y+randy]
			else:
				sample = imga[x+randx, y+randy]
			dsample = (int(sample[0]), int(sample[1]), int(sample[2])) #deep copy of color
			randintc += 1
			modifier = 1 #For taking arctic conditions into account
			if y < NORTHERN_LINE: #Disadvantage for going into north
				modifier = 0.8
			if dsample != self_color and dsample != self_borderColor and dsample in nationcolors:
				other = nationcolors[dsample]
				otherstr = other.soldierstr
				if other.starving:
					otherstr *= 0.8
				win_crit = tact_crit
				if 'tactician' in other.characteristics:
					win_crit = max(0, tact_crit - other.characteristics['tactician']/2.0) #Min chance to win is 0%
				expansion = expansion and randints[randintc%30]/max(0.1, soldierstr*modifier - otherstr + 1) <= win_crit #Won battle

			#Allow for genius saving (or disaster) throw
			randintc += 1
			if 'genius' in self_chars:
				genius = self_chars['genius']
				if genius <= 0: #Negative genius allows for much more disaster
					genius *= 2
				if randints[randintc%30] <= abs(genius)/10.0:
					expansion = genius > 0

			if not clean and not expansion: #Skip loop iteration if it's a 'dirty' pass and no expansion is needed
				continue

			surrounded = True #True if dot is surrounded by dots of same color (excluding ocean pixels)
			
			#Get all expandable pixels adjacent to border:
			for i in adj_range:
				for j in adj_range:
					newx = x+i
					newy = y+j
					#Wraparound
					if newx < 0:
						newx = maxx + newx
					elif newx > maxx:
						newx = newx - maxx
					point = (newx, newy)
					if (i == 0 and j == 0) or point in memo:
						continue
					elif newy > 0 and newy < maxy:
						color = imga[newx, newy]
						color = (int(color[0]), int(color[1]), int(color[2])) #deep copy of color
						if color != self_color and color != self_borderColor and color != OCEAN_COLOR:
							nonself_color = color
							surrounded = False
							if not expansion: #This is reached if clean is True
								break;
							if color == SEALANE_COLOR: #Expands to sealane
								imga[newx, newy] = self_borderColor
							else: #Expands on land
								if color in nationcolors: #Combat
									#Add all surrounding points into that nation's border, if the conquest point isn't in the nation's border
									opp = nationcolors[color]
									if point not in opp.border:
										opp_color = opp.color
										opp_borderColor = opp.borderColor
										opp_border_add = opp.border.add
										for a in adj_range:
											for b in adj_range:
												oppx = newx+a
												oppy = newy+b
												#Wraparound
												if oppx < 0:
													oppx = maxx + oppx
												elif oppx > maxx:
													oppx = oppx - maxx
												opppoint = (oppx, oppy)
												if oppy > 0 and oppy < maxy and (imga[oppx, oppy] == opp_color):
													opp_border_add(opppoint)
													imga[oppx, oppy] = opp_borderColor
									else:
										opp.border.discard(point)
									logistics -= 1
									opp.area.discard(point)
								imga[newx, newy] = self_borderColor
							#Change border set:
							if pixel in border:
								border_remove(pixel)
								imga[x, y] = self_color
							border_append(point)
							area_add(point)
						elif color == OCEAN_COLOR: #Can't expand to ocean
							memo_add(point) #Memoize
			if surrounded:
				border_remove(pixel)
				imga[x, y] = self_color

		#Update characteristics
		if not self.player and random.randint(0, 100) <= 5:
			self.characteristics = characteristics.addChars(self.characteristics, characteristics.createChar())

		#Update news
		if self.player and self.starving and not self.starvingnews and logistics <= 0: #Starving means previous turn, and this turn, suffered lack of logistics
			map.news.append(map.newsfont.render('Troops out of supplies!', 1, self.color))
			self.starvingnews = True
		#Update starving status
		if not self.starving and logistics <= 0:
			self.starving = True
		elif self.starving and logistics > 0:
			self.starving = False
			self.starvingnews = False

class Threads(pygame.sprite.OrderedUpdates):

	def __init__(self, w, h, y_offset, screen):
		pygame.sprite.OrderedUpdates.__init__(self)
		self.bg = pygame.Surface((w, h)).convert() #Background
		print self.bg.get_rect()
		#Game fields:
		self.birthchance = 10
		self.year = -400
		#Font:
		self.font = pygame.font.Font(None, 15)
		self.smallfont = pygame.font.Font(None, 14)
		#'Present' text:
		self.font.set_bold(True)
		self.present = []
		for letter in 'Present':
			self.present.append(self.font.render(letter, 1, pygame.Color('yellow')))
		self.font.set_bold(False)
		#Mouseover:
		self.popup_text = [] #List of popup text surfaces for blitting
		self.popup_box = None #Rect containing all popup text
		self.mousepos = None
		#Define where to draw this window
		dest_rect = self.bg.get_rect()
		dest_rect.top = y_offset
		self.dest = screen.subsurface(dest_rect)
		self.dest_rect = dest_rect
		#Make characteristics dictionary
		self.characteristics = dict()
		#Draw black background, with borders
		self.bg.fill(pygame.Color('black'))
		self.drawborder(self.bg)
		#Draw background
		self.dest.blit(self.bg, (0, 0))
		pygame.display.flip()

	def populate(self):
		self.addThread(4) #4 to make room for border
		self.addThread(4)
		for i in range(2):
			self.addThread(random.randint(8, self.bg.get_rect().width))
		self.updateChars()

	def drawborder(self, surf):
		"""surf - surface"""
		pygame.draw.rect(surf, pygame.Color('white'), self.bg.get_rect(), 2)
		pygame.draw.rect(surf, pygame.Color('grey'), self.bg.get_rect().inflate(-4, -5), 1)
		years = self.dest_rect.width-8 #Years represented by width of screen minus border
		for i in range(3, years):
			year = self.year + i/2 #(each pixel is half a year)
			if year%25 == 0: #Every 25th year, draw a mark
				pygame.draw.line(surf, pygame.Color('darkgrey'), (i, 4), (i, self.dest_rect.height-4))
				yearstr = str(year)
				if year <= 0:
					yearstr += ' BC'
					yearstr = yearstr[1:]
				else:
					yearstr += ' AD'
				text = self.font.render(yearstr, 1, pygame.Color('yellow'))
				textpos = (i+5, 5)
				surf.blit(text, textpos)
		y = 14
		for letter in self.present:
			surf.blit(letter, (5, y))
			y += letter.get_rect().h

	def draw(self):
		self.bg.fill(pygame.Color('black'))
		pygame.sprite.OrderedUpdates.draw(self, self.bg)
		self.drawborder(self.bg)
		self.dest.blit(self.bg, (0, 0))

	def drawMouseover(self, screen):
		if self.mousepos is None:
			return
		if self.popup_box is not None:
			change = 0
			if self.popup_box.left <= 0: #Obscured, move it left
				change = self.mousepos[0] - self.popup_box.left
				self.popup_box.left += change
			pygame.draw.rect(screen, pygame.Color('black'), self.popup_box)
			pygame.draw.rect(screen, pygame.Color('white'), self.popup_box, 1)
			for text, textpos in self.popup_text:
				x, y = textpos
				screen.blit(text, (x+change, y))

	def tick(self):
		#Add new threads
		if random.randint(0, 100) < self.birthchance:
			self.addThread()
		for sprite in self:
			sprite.tick()

		self.updateChars()

		#Increase time
		self.year += 2

	def updateChars(self):
		#Update characteristics
		chars = dict()
		for thread in self:
			if thread.rect.left <= 0 and not thread.death: #The thread is currently active
				chars = characteristics.addChars(chars, thread.characteristics)
		self.characteristics = chars

	def addThread(self, x=None):
		"""Returns False if addThread() failed, None if succeeded (messy, sorry)"""
		if x is None:
			x = self.bg.get_rect().width
		height = random.randint(4, self.bg.get_rect().h-15) #15 = height of border + height of thread + 1 for comfortable display
		sprite = ThreadSprite(x-4, height+11, random.randint(70, 210)) #4 for borders
		#Make sure thread doesn't collide with any other thread
		collidecount = 0
		thresh = 3 #After three or more collisions, give up
		while pygame.sprite.spritecollideany(sprite, self) and collidecount <= thresh:
			height = random.randint(4, self.bg.get_rect().h-10)
			sprite = ThreadSprite(x-4, height+11, random.randint(70, 210))
			collidecount += 1
		if collidecount <= thresh:
			self.add(sprite)
		else:
			return False

	def cutThread(self, mousepos):
		"""Cuts thread at mouse position"""
		for thread in self:
			absrect = thread.rect.move(0, self.dest_rect.top) #Sprite rect in absolute coordinates
			if absrect.collidepoint(mousepos):
				thread.startdeath(mousepos[0])
		self.updateChars()

	def mouseover(self, mousepos):
		if not self.dest_rect.collidepoint(mousepos): #Not in threads window, don't bother
			self.popup_text = [] #Clear popup text
			self.popup_box = None
			return
		for thread in self:
			absrect = thread.rect.move(0, self.dest_rect.top) #Sprite rect in absolute coordinates
			if absrect.collidepoint(mousepos):
				self.popup(thread, mousepos)
				return
		self.popup_text = [] #No thread moused over, clear popup text
		self.popup_box = None
		self.mousepos = mousepos

	def popup(self, thread, mousepos):
		x = mousepos[0] #*RIGHTMOST* x of popup
		y = self.dest_rect.top + thread.rect.top - len(thread.characteristics.keys())*self.smallfont.get_height() #Topmost y of popup
		y_offset = 0
		self.popup_text = []
		maxw = 0
		maxh = 0
		chars = thread.characteristics.items()
		#Sort characteristics by value
		chars.sort(lambda x, y: (y[1] - x[1], x[1] - y[1])[x[1] < 0 and y[1] < 0])
		for char, val in chars:
			textstr = char+': '+str(val)+'%'
			text = self.smallfont.render(textstr, 1, pygame.Color('white'))
			w, h = text.get_rect().size
			textpos = (x-w, y+y_offset)
			self.popup_text.append((text, textpos))
			y_offset += h
			maxw = max(maxw, w)
		self.popup_box = pygame.rect.Rect(x-maxw-1, y-1, maxw+2, y_offset+2) #-1 to allow for border

class ThreadSprite(pygame.sprite.Sprite):
	
	def __init__(self, x_offset, y_offset, length):
		"""x_offset = offset of x coordinate from left
y_offset = offset of y from top
length = length of string, in pixels"""
		pygame.sprite.Sprite.__init__(self)
		self.length = length
		#Pick color:
		badcolor = True #Initially true (for the do-while loop below), this will be false
		#or true depending on how close the color is to black or white
		while badcolor:
			self.color = pygame.Color(random.choice(pygame.color.THECOLORS.keys()))
			badcolor = math.sqrt(math.pow(self.color[0], 2)+math.pow(self.color[1], 2)+math.pow(self.color[2], 2)) <= 50 or math.sqrt(math.pow(self.color[0] - 255, 2)+math.pow(self.color[1] - 255, 2)+math.pow(self.color[2] - 255, 2)) <= 50
		self.image = pygame.Surface((length, 6)).convert()
		self.rect = self.image.get_rect()
		self.rect.topleft = (x_offset, y_offset)
		self.death = False #False = not playing cut animation, True otherwise
		self.deathlength = 1 #Length of black bar indicating death
		self.cutx = None #x-position at which thread is cut (in terms of self.rect)
		#Get characteristics:
		self.characteristics = characteristics.createChar()

		#Draw thread on image
		self.image.fill(pygame.Color('black')) #Black out the entire thread (background color)
		#Draw thread as line (with width = height of thread)
		pygame.draw.line(self.image, self.color, (0, 0), (self.length, 0), self.rect.h*2)
		#Draw bright center
		innerh = self.rect.h/2-2 #Height of top of inner rectangle
		pygame.draw.line(self.image, pygame.color.add(self.color, 20), (0, innerh), (self.length, innerh), 2)
		#Draw little thread stripes to show movement
		angle = math.radians(random.randint(30, 75)) #Angle of stripe, measured from vertical
		xshift = self.rect.h*math.sin(angle) #Amount by which x shifts at end of stripe compared to start of stripe
		stripecolor = pygame.color.subtract(self.color, 40)
		for i in range(0, length, random.randint(3, 5)): #Make stripes at random intervals
			p1 = (i, 0)
			p2 = (i+xshift, self.rect.h)
			pygame.draw.line(self.image, stripecolor, p1, p2)

	def tick(self):
		self.rect.move_ip(-4, 0)
		if self.rect.right <= 0:
			self.kill()

	def update(self):
		if self.death:
			minx = self.cutx - self.deathlength
			maxx = self.cutx + self.deathlength
			if minx <= 0 or maxx >= self.rect.width: #Dead
				self.kill()
			else: #Keep on drawing death animation
				pygame.draw.line(self.image, (0, 0, 0), (self.cutx-self.deathlength, 0), (self.cutx+self.deathlength, 0), self.rect.h*2)
			self.deathlength *= 2

	def startdeath(self, absx):
		"""Queue death animation for play"""
		self.death = True
		self.cutx = absx - self.rect.left
